<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 07/10/2004 *
 * ===================                          Last change: 07/10/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : register_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for register extension         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktion fuer register-Erweiterung     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2013 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Checks whether all required registration fields are set
function ifRequiredRegisterFieldsAreSet (&$array) {
	// By default all is fine
	$ret = TRUE;
	foreach ($array as $key => $value) {
		// Check all fields that must register
		$result = sqlQueryEscaped("SELECT `id` FROM `{?_MYSQL_PREFIX?}_must_register` WHERE `field_name`='%s' AND `field_required`='Y' LIMIT 1",
			array($key), __FUNCTION__, __LINE__);

		// Entry found?
		if (sqlNumRows($result) == 1) {
			// Check if extension country is not found (you have to enter the 2-chars long country code) or
			// if extensions is present check if country code was selected
			//         01              2         21    12             3         32    234     5      54    4               43    34                      4    4      5      5432    2      3                      3210
			$country = ((!isExtensionActive('country')) || ((isExtensionActive('country')) && (((empty($value)) && ($key == 'cntry')) || (($key == 'country_code') && (!empty($value)))) && (!empty($array['country_code']))));
			if ((empty($value)) && ($country === FALSE)) {
				// Required field not set
				$array[$key] = '!';
				$ret = FALSE;
			} // END - if
		} // END - if

		// Free result
		sqlFreeResult($result);
	} // END - foreach

	// Return result
	return $ret;
}

// Generates a 'category table' for the registration form
function registerGenerateCategoryTable ($mode, $configEntry = 'register_default') {
	// Init output
	$OUT = '';

	/*
	 * Guests are mostly not interested in how many members has choosen an
	 * individual category.
	 */
	$whereStatement = "WHERE `visible`='Y' ";

	// Admins are allowed to see every category...
	if (isAdmin()) {
		$whereStatement = '';
	} // END - if

	// Look for categories
	$result = sqlQuery('SELECT
	`id`,
	`cat`,
	`visible`
FROM
	`{?_MYSQL_PREFIX?}_cats`
' . $whereStatement . '
ORDER BY
	`sort` ASC',
		__FUNCTION__, __LINE__);

	if (!ifSqlHasZeroNums($result)) {
		// List alle visible modules (or all to the admin)
		$OUT .= '<table border="0" cellspacing="0" cellpadding="0" width="100%">';
		while ($content = sqlFetchArray($result)) {
			// Prepare array for the template
			$content['default_yes'] = '';
			$content['default_no']  = '';

			// Mark categories
			if ((postRequestElement('cat', $content['id']) == 'Y') || ((getConfig($configEntry) == 'Y') && (!isPostRequestElementSet('cat', $content['id'])))) {
				$content['default_yes'] = ' checked="checked"';
			} else {
				$content['default_no']  = ' checked="checked"';
			}

			// Load template and switch color
			$OUT .= loadTemplate('guest_cat_row', TRUE, $content);
		} // END - while
		$OUT .= '</table>';

		// Free memory
		sqlFreeResult($result);
	} else {
		// No categories setted up so far...
		$OUT .= displayMessage('{--NO_CATEGORIES_VISIBLE--}', TRUE);
	}

	// Return generated HTML code
	return $OUT;
}

// Outputs a 'failed message'
function registerOutputFailedMessage ($messageId, $extra='') {
	if (empty($messageId)) {
		outputHtml('<div class="bad">' . $extra . '</div>');
	} else {
		outputHtml('<div class="bad">{--' . $messageId . '--}' . $extra . '</div>');
	}
}

// Checks whether the registration data is complete
function isRegistrationDataComplete () {
	// Init elements
	$GLOBALS['registration_ip_timeout']    = FALSE;
	$GLOBALS['registration_weak_password'] = FALSE;
	$GLOBALS['registration_selected_cats'] = '0';

	// Default is okay
	$isOkay = TRUE;
	$isRandom = FALSE;

	// First we only check the submitted data then we continue... :)
	//
	// Did he agree to the terms of usage?
	if (postRequestElement('agree') != 'Y') {
		setPostRequestElement('agree', '!');
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'agree=N - User did not agree with terms of usage.');
		$isOkay = FALSE;
	} // END - if

	// Did he enter a valid email address? (we really don't care about
	// that, he has to click on a confirmation link :P )
	if ((!isAdmin()) && ((!isPostRequestElementSet('email')) || (!isEmailValid(postRequestElement('email'))))) {
		setPostRequestElement('email', '!');
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter proper email address.');
		$isOkay = FALSE;
	} // END - if

	// And what about surname and family's name?
	if (!isPostRequestElementSet('surname')) {
		setPostRequestElement('surname', '!');
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter surname.');
		$isOkay = FALSE;
	} // END - if
	if (!isPostRequestElementSet('family')) {
		setPostRequestElement('family', '!');
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter family name.');
		$isOkay = FALSE;
	} // END - if

	// Get temporary array for modification
	$postArray = postRequestArray();

	// Check for required fields
	$isOkay = ($isOkay && ifRequiredRegisterFieldsAreSet($postArray));

	// Set it back in request
	setPostRequestArray($postArray);

	// Are both passwords zero length?
	if ((strlen(postRequestElement('password1')) == 0) && (strlen(postRequestElement('password2')) == 0) && ($isOkay === TRUE)) {
		// Is the extension 'register' newer or equal 0.5.5?
		if ((isExtensionInstalledAndNewer('register', '0.5.5')) && (isRegisterGeneratePasswordEmptyEnabled())) {
			// Generate a random password
			$randomPassword = generatePassword();
			$isRandom = TRUE;

			// Set it in both entries
			setPostRequestElement('password1', $randomPassword);
			setPostRequestElement('password2', $randomPassword);
		} else {
			// Not allowed or no recent extension version
			setPostRequestElement('password1', '!');
			setPostRequestElement('password2', '!');

			// ... which is both not okay
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Random password generation not possible, isExtensionInstalledAndNewer(register, 0.5.5)=' . intval(isExtensionInstalledAndNewer('register', '0.5.5')) . ',isRegisterGeneratePasswordEmptyEnabled()=' . intval(isRegisterGeneratePasswordEmptyEnabled()));
			$isOkay = FALSE;
		}
	} // END - if

	// Did he enter his password twice?
	if (((!isPostRequestElementSet('password1')) || (!isPostRequestElementSet('password2'))) || ((postRequestElement('password1') != postRequestElement('password2')) && (isPostRequestElementSet('password1')) && (isPostRequestElementSet('password2')))) {
		if ((postRequestElement('password1') != postRequestElement('password2')) && (isPostRequestElementSet('password1')) && (isPostRequestElementSet('password2'))) {
			// Both passwords did not match
			setPostRequestElement('password1', '!');
			setPostRequestElement('password2', '!');
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter same passwords.');
		} else {
			if (!isPostRequestElementSet('password1')) {
				// Password 1 is empty
				setPostRequestElement('password1', '!');
				//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter password1.');
			} else {
				// Password 2 is empty
				setPostRequestElement('password1', '');
			}
			if (!isPostRequestElementSet('password2')) {
				// Password 2 is empty
				setPostRequestElement('password2', '!');
				//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did not enter password2.');
			} else {
				// Password 1 is empty
				setPostRequestElement('password2', '');
			}
		}
		$isOkay = FALSE;
	} // END - if

	// Is the password strong enough?
	if (($isRandom === FALSE) && (!isStrongPassword(postRequestElement('password1')))) {
		$GLOBALS['registration_weak_password'] = TRUE;
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did enter a short password.');
		$isOkay = FALSE;
	} // END - if

	// Do this check only when no admin is logged in
	if (ifPostContainsSelections('cat')) {
		// Only continue with array
		foreach (postRequestElement('cat') as $id => $answer) {
			// Is this category choosen?
			if ($answer == 'Y') {
				$GLOBALS['registration_selected_cats']++;
			} // END - if
		} // END - foreach
	} // END - if

	// Enougth categories selected?
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'isOkay=' . intval($isOkay) . ',selected=' . $GLOBALS['registration_selected_cats'] . '/' . getLeastCats());
	$isOkay = (($isOkay) && ($GLOBALS['registration_selected_cats'] >= getLeastCats()));

	// Check if email is taken, if configured
	if ((isExtensionInstalledAndNewer('other', '0.3.0')) && (isCheckDoubleEmailEnabled()) && (postRequestElement('email') != '!') && (isEmailTaken(postRequestElement('email'))) && (!isAdmin())) {
		// Is already used
		setPostRequestElement('email', '?');
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'User did enter a already used email address.');
		$isOkay = FALSE;
	} // END - if

	// Check for IP timeout?
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'isOkay=' . intval($isOkay));
	if ((!isAdmin()) && (getIpTimeout() > 0)) {
		// Check his IP number
		$GLOBALS['registration_ip_timeout'] = (countSumTotalData(detectRemoteAddr()  , 'user_data', 'userid', 'REMOTE_ADDR', TRUE, ' AND ((UNIX_TIMESTAMP() - `joined`) < {?ip_timeout?} OR (UNIX_TIMESTAMP() - `last_update`) < {?ip_timeout?})') == 1);
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'isOkay=' . intval($isOkay).',timeout='.intval($GLOBALS['registration_ip_timeout']));
		$isOkay = (($isOkay) && (!$GLOBALS['registration_ip_timeout']));
	} // END - if

	// Return result
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'isOkay=' . intval($isOkay) . ' - EXIT!');
	return $isOkay;
}

// Do the registration
function doUserRegistration () {
	// Do not register an account on absent ext-user
	if (!isExtensionInstalled('user')) {
		// Please report this
		reportBug(__FUNCTION__, __LINE__, 'Tried to register a user account without ext-user installed.');
	} // END - if

	// Init filter data array
	$filterData = array(
		// Registration status is always FALSE by default
		'status' => FALSE,
	);

	// Run filter chain for user registration
	$filterData = runFilterChain('user_registration', $filterData);

	// Return status
	return $filterData['status'];
}

// Generic user registration
function doGenericUserRegistration () {
	// Init extra SQL data
	initExtraRegistrationSql();

	// Init filter data
	$filterData = array(
		// Initialization not done by default
		'init_done'   => FALSE,
		'post_data'   => postRequestArray(),
		'blacklisted' => '',
		'message'     => '{--PRE_USER_REGISTRATION_FAILED--}',
	);

	// Run the pre-registration chain
	$filterData = runFilterChain('pre_user_registration', $filterData);

	// Did the initialization work?
	if ($filterData['init_done'] === FALSE) {
		// Something bad happened!
		displayMessage($filterData['message']);

		// Stop here
		return FALSE;
	} // END - if

	// Create user's account...
	sqlQueryEscaped("INSERT INTO
	`{?_MYSQL_PREFIX?}_user_data`
(
	`gender`,
	`surname`,
	`family`,
	`street_nr`,
	%s,
	`zip`,
	`city`,
	`email`,
	`birth_day`,
	`birth_month`,
	`birth_year`,
	`password`,
	`max_mails`,
	`receive_mails`,
	`refid`,
	`status`,
	`user_hash`,
	`REMOTE_ADDR`,
	`joined`,
	`last_update`,
	`ref_payout`
	" . $GLOBALS['register_sql_columns'] . "
) VALUES (
	'%s',
	'%s',
	'%s',
	'%s',
	'%s',
	%s,
	'%s',
	'%s',
	%s,
	%s,
	%s,
	'%s',
	%s,
	%s,
	%s,
	'%s',
	'%s',
	'{%%pipe,detectRemoteAddr%%}',
	UNIX_TIMESTAMP(),
	UNIX_TIMESTAMP(),
	{?ref_payout?}
	" . $GLOBALS['register_sql_data'] . "
)",
	array(
		$GLOBALS['register_country_row'],
		substr(postRequestElement('gender'), 0, 1),
		postRequestElement('surname'),
		postRequestElement('family'),
		postRequestElement('street_nr'),
		$GLOBALS['register_country_data'],
		bigintval(postRequestElement('zip')),
		postRequestElement('city'),
		postRequestElement('email'),
		bigintval(postRequestElement('day')),
		bigintval(postRequestElement('month')),
		bigintval(postRequestElement('year')),
		generateHash(postRequestElement('password1')),
		bigintval(postRequestElement('max_mails')),
		bigintval(postRequestElement('max_mails')),
		convertZeroToNull(postRequestElement('refid')),
		postRequestElement('status'),
		$GLOBALS['register_confirm_hash']
	), __FUNCTION__, __LINE__);

	// Get his userid
	$filterData['register_insert_id'] = getSqlInsertId();

	// Did this work?
	if (!isValidId($filterData['register_insert_id'])) {
		// Something bad happened!
		displayMessage('{--USER_NOT_REGISTERED--}');

		// Stop here
		return FALSE;
	} // END - if

	// Shall we reset random refid? Only possible with latest ext-user
	if (isExtensionInstalledAndNewer('user', '0.3.4')) {
		// Reset all accounts, registration is done
		sqlQuery('UPDATE `{?_MYSQL_PREFIX?}_user_data` SET `rand_confirmed`=0', __FUNCTION__, __LINE__);
	} // END - if

	// Update referral table
	updateReferralCounter($filterData['register_insert_id']);

	// Write his welcome-points
	initReferralSystem();
	addPointsThroughReferralSystem(
		// Subject
		'register_welcome',
		// User's id number
		$filterData['register_insert_id'],
		// Points to add
		getPointsRegister(),
		// Referral id (or NULL if none set)
		convertZeroToNull(postRequestElement('refid'))
	);

	// Write catgories
	if (ifPostContainsSelections('cat')) {
		// Init SQL
		$sql = 'INSERT INTO `{?_MYSQL_PREFIX?}_user_cats` (`userid`, `cat_id`) VALUES';

		// Write all entries
		foreach (postRequestElement('cat') as $categoryId => $joined) {
			// "Join" this group?
			if ($joined == 'Y') {
				// Insert category entry
				$sql .= ' (' . $filterData['register_insert_id'] . ', ' . bigintval($categoryId) . '),';
			} // END - if
		} // END - foreach

		// Run SQL without last commata
		sqlQuery(substr($sql, 0, -1), __FUNCTION__, __LINE__);
	} // END - if

	// Registration phase is done here, so for tester accounts we end here
	if (((getExtensionVersion('user') >= '0.5.0')) && (isTesterUserName(postRequestElement('surname'))) && (ifTesterAccountsAllowed())) {
		// All fine here
		return TRUE;
	} // END - if

	// ... rewrite a zero referral id to the main title
	if (!isValidId(postRequestElement('refid'))) {
		setPostRequestElement('refid', getMainTitle());
	} // END - if

	// Is ZIP code set?
	if (isPostRequestElementSet('zip')) {
		// Prepare data array for the email template
		$content = array(
			'hash'     => $GLOBALS['register_confirm_hash'],
			'userid'   => $filterData['register_insert_id'],
			'gender'   => sqlEscapeString(postRequestElement('gender')),
			'surname'  => sqlEscapeString(postRequestElement('surname')),
			'family'   => sqlEscapeString(postRequestElement('family')),
			'email'    => sqlEscapeString(postRequestElement('email')),
			'street'   => sqlEscapeString(postRequestElement('street_nr')),
			'city'     => sqlEscapeString(postRequestElement('city')),
			'zip'      => bigintval(postRequestElement('zip')),
			'country'  => $GLOBALS['register_country_data'],
			'refid'    => sqlEscapeString(postRequestElement('refid')),
			'password' => sqlEscapeString(postRequestElement('password1')),
		);
	} else {
		// No ZIP code entered
		$content = array(
			'hash'     => $GLOBALS['register_confirm_hash'],
			'userid'   => $filterData['register_insert_id'],
			'gender'   => sqlEscapeString(postRequestElement('gender')),
			'surname'  => sqlEscapeString(postRequestElement('surname')),
			'family'   => sqlEscapeString(postRequestElement('family')),
			'email'    => sqlEscapeString(postRequestElement('email')),
			'street'   => sqlEscapeString(postRequestElement('street_nr')),
			'city'     => sqlEscapeString(postRequestElement('city')),
			'zip'      => '',
			'country'  => $GLOBALS['register_country_data'],
			'refid'    => sqlEscapeString(postRequestElement('refid')),
			'password' => sqlEscapeString(postRequestElement('password1')),
		);
	}

	// Continue with birthday...
	switch (getLanguage()) {
		case 'de':
			$content['birthday'] = bigintval(postRequestElement('day')) . '.' . bigintval(postRequestElement('month')) . '.' . bigintval(postRequestElement('year'));
			break;

		default:
			$content['birthday'] = bigintval(postRequestElement('month')) . '/' . bigintval(postRequestElement('day')) . '/' . bigintval(postRequestElement('year'));
			break;
	} // END - switch

	// Display information to the user that he got mail and send it away
	$messageGuest = loadEmailTemplate('guest_register_done', $content, $filterData['register_insert_id'], FALSE);

	// Send mail to user (confirmation link!)
	sendEmail($filterData['register_insert_id'], '{--GUEST_CONFIRM_LINK_SUBJECT--}', $messageGuest);

	// Send mail to admin
	sendAdminNotification('{--ADMIN_NEW_ACCOUNT_SUBJECT--}', 'admin_register_done', $content, $filterData['register_insert_id']);

	// All fine
	return TRUE;
}

// Initialize extra registration SQL
function initExtraRegistrationSql () {
	$GLOBALS['register_sql_columns'] = '';
	$GLOBALS['register_sql_data']    = '';
}

// Add extra column for registration SQL
function addExtraRegistrationColumns ($column) {
	// Add column
	$GLOBALS['register_sql_columns'] .= $column;
}

// Add extra data for registration SQL
function addExtraRegistrationData ($data) {
	// Add column
	$GLOBALS['register_sql_data'] .= $data;
}

//-----------------------------------------------------------------------------
//                      Wrapper functions for ext-register
//-----------------------------------------------------------------------------

// Getter for 'display_refid'
function getDisplayRefid () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = getConfig('display_refid');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Checks whether 'display_refid' is "Y"
function isDisplayRefidEnabled () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = (getDisplayRefid() == 'Y');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Getter for 'ip_timeout'
function getIpTimeout () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = getConfig('ip_timeout');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Getter for 'register_default'
function getRegisterDefault () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = getConfig('register_default');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Checks whether 'register_default' is "YES"
function isRegisterDefaultEnabled () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = (getRegisterDefault() == 'Y');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Getter for 'register_generate_password_empty'
function getRegisterGeneratePasswordEmpty () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = getConfig('register_generate_password_empty');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Checks whether 'register_generate_password_empty' is "YES"
function isRegisterGeneratePasswordEmptyEnabled () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = (getRegisterGeneratePasswordEmpty() == 'Y');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// Getter for 'default_registration_provider'
function getDefaultRegistrationProvider () {
	// Is the cache entry set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// No, so determine it
		$GLOBALS[__FUNCTION__] = getConfig('default_registration_provider');
	} // END - if

	// Return cached entry
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for least_cats
function getLeastCats () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('least_cats');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// ----------------------------------------------------------------------------
//                            Template helper functions
// ----------------------------------------------------------------------------

// Template helper for generating a category selection table for admin area with given configuration entry
function doTemplateAdminRegisterCategoryTable ($templateName, $clear = FALSE, $configEntry) {
	// Call the inner function
	return registerGenerateCategoryTable('admin', $configEntry);
}

// Template helper for generating a list of all activated user registration provider
function doTemplateGuestRegistrationList ($templateName, $clear = FALSE) {
	// Init output
	$content = '';

	// Default is only activated provider
	$addSql = " AND `provider_is_active`='Y'";

	// Is admin logged-in?
	if (isAdmin()) {
		// Then show all
		$addSql = '';
	} // END - if

	// Search for all
	$result = sqlQuery("SELECT
	`provider_name`,
	`provider_extension`
FROM
	`{?_MYSQL_PREFIX?}_user_register_provider`
WHERE
	`provider_extension` != 'register'
	" . $addSql . "
ORDER BY
	`provider_name` ASC", __FUNCTION__, __LINE__);

	// Are there entries?
	if (sqlNumRows($result) > 0) {
		// Loop through all
		$row = '';
		while ($content = sqlFetchArray($result)) {
			// Load row template
			$row .= loadTemplate('guest_registration_provider_row', TRUE, $content);
		} // END - while

		// Load main template
		$content = loadTemplate('guest_registration_provider', TRUE, $row);
	} else {
		// Nothing found
		$content = returnMessage('{--GUEST_EXTRA_REGISTRATION_PROVIDER_404--}');
	}

	// Free result
	sqlFreeResult($result);

	// Return the generated content
	return $content;
}

// ----------------------------------------------------------------------------
//                            "Translator" functions
// ----------------------------------------------------------------------------

function translateRegistrationProviderName ($providerName) {
	// "Translate it"
	return '{--REGISTRATION_PROVIDER_' . strtoupper($providerName) . '--}';
}

// [EOF]
?>
